/*
 * Copyright (C) 2009, Broadcom Corporation
 * All Rights Reserved.
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: time.c,v 1.34 2009-07-17 05:05:33 Exp $
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/serial_reg.h>
#include <linux/interrupt.h>
#include <asm/addrspace.h>
#include <asm/io.h>
#include <asm/time.h>

#include <typedefs.h>
#include <osl.h>
#include <bcmutils.h>
#include <bcmnvram.h>
#include <sbchipc.h>
#include <siutils.h>
#include <hndmips.h>
#include <mipsinc.h>
#include <hndcpu.h>
#include <bcmdevs.h>

#include <wps_led.h> 

/* Global SI handle */
extern si_t *bcm947xx_sih;
extern spinlock_t bcm947xx_sih_lock;

/* Convenience */
#define sih bcm947xx_sih
#define sih_lock bcm947xx_sih_lock
#define WATCHDOG_MIN	3000	/* milliseconds */

extern int panic_timeout;
static uint32 watchdog = 0;


#define LED_BLINK_RATE_NORMAL   50
#define LED_BLINK_RATE_QUICK    10

int wps_led_pattern = 0;
int wps_led_state = 0;


int is_wl_secu_mode = 0;
static int wps_led_is_on = 0;

static int wps_led_state_old = 1;

static si_t *gpio_sih;
static int wps_led_init(void)
{
    if (!(gpio_sih = si_kattach(SI_OSH))) 
    {
        printk("%s failed!\n", __FUNCTION__);
        return -ENODEV;
    }

    return 0;
}

static int gpio_led_on_off(int gpio_num, int on_off)
{
    si_gpioreserve(gpio_sih, 1 << WPS_LED_GPIO, GPIO_APP_PRIORITY);
    si_gpioouten(gpio_sih, 1 << WPS_LED_GPIO, 1 << WPS_LED_GPIO, GPIO_APP_PRIORITY);
    si_gpioout(gpio_sih, 1 << WPS_LED_GPIO, on_off << WPS_LED_GPIO, GPIO_APP_PRIORITY);
    
    wps_led_is_on = on_off;
    
    return 0;
}


#ifdef WLAN_REPEATER
int wps_led_on_off(int on_off)
{
    si_gpioreserve(gpio_sih, 1 << WPS_LED_GPIO, GPIO_APP_PRIORITY);
    si_gpioouten(gpio_sih, 1 << WPS_LED_GPIO, 1 << WPS_LED_GPIO, GPIO_APP_PRIORITY);
    si_gpioout(gpio_sih, 1 << WPS_LED_GPIO, on_off << WPS_LED_GPIO, GPIO_APP_PRIORITY);
    
    return 0;
}
#endif




static void quick_blink2(void)
{
    static int interrupt_count = -1;

    interrupt_count++;
    if (interrupt_count == LED_BLINK_RATE_QUICK * 2)
        interrupt_count = 0;
    
    if (interrupt_count == 0)
        gpio_led_on_off(1, 1);
    else if (interrupt_count == LED_BLINK_RATE_QUICK)
        gpio_led_on_off(1, 0);
}

static void quick_blink(void)
{
    
    //static int blink_interval = 3000; /* 30 seconds */
    static int blink_interval = 500; /* 5 seconds */
    
    static int interrupt_count = -1;

    blink_interval--;
    interrupt_count++;
    if (interrupt_count == LED_BLINK_RATE_QUICK * 2)
        interrupt_count = 0;
    
    if (interrupt_count == 0)
        gpio_led_on_off(1, 1);
    else if (interrupt_count == LED_BLINK_RATE_QUICK)
        gpio_led_on_off(1, 0);
        
    if ( blink_interval <= 0 )
    {
        
        //blink_interval = 3000;
        blink_interval = 500;
        
        wps_led_state = 0;
    }
}

static int normal_blink(void)
{
    static int interrupt_count = -1;

    interrupt_count++;
    if (interrupt_count == LED_BLINK_RATE_NORMAL * 2)
        interrupt_count = 0;
    
    if (interrupt_count == 0)
        gpio_led_on_off(1, 1);
    else if (interrupt_count == LED_BLINK_RATE_NORMAL)
        gpio_led_on_off(1, 0);
}
static int wps_ap_lockdown_blink(void)
{
    static int interrupt_count = -1;

    interrupt_count++;
    if (interrupt_count == LED_BLINK_RATE_QUICK * 10)
        interrupt_count = 0;
    
    if (interrupt_count == 0)
        gpio_led_on_off(1, 1);
    else if (interrupt_count == LED_BLINK_RATE_QUICK)
        gpio_led_on_off(1, 0);
}


/* Add USB LED  */
#if (defined INCLUDE_USB_LED)
#define GPIO_USB_LED    27
#define LED_BLINK_RATE  5
int usb_pkt_cnt = 0;
int usb_led_state = 0;
static int usb_led_state_old = 1;
EXPORT_SYMBOL(usb_pkt_cnt);
EXPORT_SYMBOL(usb_led_state);

static int gpio_on_off(int gpio_num, int on_off)
{
    si_gpioreserve(gpio_sih, 1 << gpio_num, GPIO_APP_PRIORITY);
    si_gpioouten(gpio_sih, 1 << gpio_num, 1 << gpio_num, GPIO_APP_PRIORITY);
    si_gpioout(gpio_sih, 1 << gpio_num, on_off << gpio_num, GPIO_APP_PRIORITY);
    return 0;
}

static int usb_normal_blink(void)
{
    static int interrupt_count = -1;
    static int usb_pkt_cnt_old = 0;

    interrupt_count++;
    if (interrupt_count == LED_BLINK_RATE * 2)
        interrupt_count = 0;
    
    if (interrupt_count == 0)
        gpio_on_off(GPIO_USB_LED, 1);
    else if (interrupt_count == LED_BLINK_RATE)
    {
        if (usb_pkt_cnt != usb_pkt_cnt_old) 
        {
            usb_pkt_cnt_old = usb_pkt_cnt;
            gpio_on_off(GPIO_USB_LED, 0);
        }
    }
    return 0;
}
#endif



#if (defined INCLUDE_WNR2200_WLAN_LED_TEMP)
static int enable_wlan_led = 1;
#define WLAN_IF_NAME    "eth1"
#define GPIO_WLAN_LED   23
#include "wlioctl.h"
#ifndef WLC_GET_PKT_CNT
#define WLC_GET_PKT_CNT 137
#endif
int wlan_led_state = 0;
int wlan_led_state_old = 1;
EXPORT_SYMBOL(wlan_led_state);
static int wlan_normal_blink(void)
{
    struct net_device *wlan_dev1 = NULL;
    unsigned long wlan_pkt_cnt = 0;
    static unsigned long wlan_pkt_cnt_old = 0;
    static int interrupt_count = 0;

    interrupt_count++;
    if (interrupt_count == LED_BLINK_RATE * 2)
        interrupt_count = 0;
    
    if (interrupt_count == 0)
        gpio_on_off(GPIO_WLAN_LED, 1);
    else if (interrupt_count == LED_BLINK_RATE)
    {
        wlan_dev1 = dev_get_by_name(WLAN_IF_NAME);
        if (wlan_dev1) 
        {
            wl_ioctl_t ioc;
            get_pktcnt_t pkt_cnt;
            struct ifreq ifr;
            int ret;

            ioc.cmd = WLC_GET_PKT_CNT;
            ioc.buf = (void *)(&pkt_cnt);
            ioc.len = sizeof(pkt_cnt);
            strcpy(ifr.ifr_name, WLAN_IF_NAME);
            ifr.ifr_data = (caddr_t) &ioc;
            ret = wlan_dev1->do_ioctl(wlan_dev1, &ifr, SIOCDEVPRIVATE);
            if (!ret) 
            {
                //wlan_pkt_cnt = pkt_cnt.rx_good_pkt + pkt_cnt.rx_bad_pkt +
                //               pkt_cnt.tx_good_pkt + pkt_cnt.tx_bad_pkt;
                wlan_pkt_cnt = pkt_cnt.rx_good_pkt + pkt_cnt.tx_good_pkt;
            }
            dev_put(wlan_dev1);
        }
    
        if (wlan_pkt_cnt != wlan_pkt_cnt_old)
        {
            wlan_pkt_cnt_old = wlan_pkt_cnt;
            gpio_on_off(GPIO_WLAN_LED, 0);
        }
    }
    return 0;
}
#endif


void __init
bcm947xx_time_init(void)
{
	unsigned int hz;
	char cn[8];

	/*
	 * Use deterministic values for initial counter interrupt
	 * so that calibrate delay avoids encountering a counter wrap.
	 */
	write_c0_count(0);
	write_c0_compare(0xffff);

	if (!(hz = si_cpu_clock(sih)))
		hz = 100000000;

	bcm_chipname(sih->chip, cn, 8);
	printk("CPU: BCM%s rev %d at %d MHz\n", cn, sih->chiprev,
	       (hz + 500000) / 1000000);

	/* Set MIPS counter frequency for fixed_rate_gettimeoffset() */
	mips_counter_frequency = hz / 2;

	/* Set watchdog interval in ms */
	watchdog = simple_strtoul(nvram_safe_get("watchdog"), NULL, 0);

	/* Ensure at least WATCHDOG_MIN */
	if ((watchdog > 0) && (watchdog < WATCHDOG_MIN))
		watchdog = WATCHDOG_MIN;

	/* Set panic timeout in seconds */
	panic_timeout = watchdog / 1000;
    /* Init WLAN LED */
    wps_led_init(); 

#if (defined INCLUDE_WNR2200_WLAN_LED_TEMP)
    if (nvram_match("enable_wlan_led", "0"))
        enable_wlan_led = 0;
#endif
}

#ifdef CONFIG_HND_BMIPS3300_PROF
extern bool hndprofiling;
#ifdef CONFIG_MIPS64
typedef u_int64_t sbprof_pc;
#else
typedef u_int32_t sbprof_pc;
#endif
extern void sbprof_cpu_intr(sbprof_pc restartpc);
#endif	/* CONFIG_HND_BMIPS3300_PROF */

static void
bcm947xx_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
#ifdef CONFIG_HND_BMIPS3300_PROF
	/*
	 * Are there any ExcCode or other mean(s) to determine what has caused
	 * the timer interrupt? For now simply stop the normal timer proc if
	 * count register is less than compare register.
	 */
	if (hndprofiling) {
		sbprof_cpu_intr(regs->cp0_epc +
		                ((regs->cp0_cause >> (CAUSEB_BD - 2)) & 4));
		if (read_c0_count() < read_c0_compare())
			return;
	}
#endif	/* CONFIG_HND_BMIPS3300_PROF */

	/* Generic MIPS timer code */
	timer_interrupt(irq, dev_id, regs);

	/* Set the watchdog timer to reset after the specified number of ms */
	if (watchdog > 0)
		si_watchdog_ms(sih, watchdog);

#ifdef	CONFIG_HWSIM
	(*((int *)0xa0000f1c))++;
#endif

    
    /* Blink LED depending of WPS status */
    if ( wps_led_state == 0 )
    {
        if (wps_led_state_old != 0)
            gpio_led_on_off(1, 0);

        
#ifdef WLAN_REPEATER
        if (!is_wl_secu_mode)
#else
        if ((!is_wl_secu_mode) && wps_led_is_on)
#endif
            gpio_led_on_off(1, 0);

#ifdef WLAN_REPEATER
        if (is_wl_secu_mode)
#else
        if (is_wl_secu_mode && (!wps_led_is_on))
#endif
            gpio_led_on_off(1, 1);
        
    }
    else
    if (wps_led_state == 1)
    {
        normal_blink();
    }
    else
    if (wps_led_state == 2)
    {
        quick_blink();
    }
    else
    if (wps_led_state == 3)
    {
        quick_blink2();
    }
    else
    if (wps_led_state == 4)
    {
        wps_ap_lockdown_blink();
    }
    wps_led_state_old = wps_led_state;
        

    
    /* Blink USB LED if necessary */
#if (defined INCLUDE_USB_LED)
    if (usb_led_state)
    {
        usb_normal_blink();
    }
    else
    {
        if (usb_led_state_old)
            gpio_on_off(GPIO_USB_LED, 0);
    }
    usb_led_state_old = usb_led_state;
#endif
    

    
    /* Temp solution for WLAN LED (should use sb/1/ledbhxx for WLAN LED) */
#if (defined INCLUDE_WNR2200_WLAN_LED_TEMP)
    if (enable_wlan_led)
    {
        if (wlan_led_state)
            wlan_normal_blink();
        else
        if (wlan_led_state_old)
            gpio_on_off(GPIO_WLAN_LED, 0);
        wlan_led_state_old = wlan_led_state;
    }
#endif
    
}

static struct irqaction bcm947xx_timer_irqaction = {
	bcm947xx_timer_interrupt,
	SA_INTERRUPT,
	0,
	"timer",
	NULL,
	NULL
};

void __init
bcm947xx_timer_setup(struct irqaction *irq)
{
	/* Enable the timer interrupt */
	setup_irq(7, &bcm947xx_timer_irqaction);
}
